@@ -2,6 +2,8 @@
 from __future__ import print_function
 
 # standard lib
+import json
+import math
 import os
 import sys
 import random
@@ -50,7 +52,7 @@ class Collection(object):
             self,
             records=None,
             input_dir=None,
-            trees_dir=None,
+            param_dir=None,
             file_format='fasta',
             compression=None,
     ):
@@ -58,10 +60,10 @@ class Collection(object):
         self._records = None
         self._input_files = None
 
-        if records:
+        if records is not None:
             self.records = records
 
-        elif input_dir:
+        elif input_dir is not None:
             directorycheck(input_dir)
             optioncheck(file_format, ['fasta', 'phylip'])
             self.records = self.read_alignments(input_dir,
@@ -72,8 +74,8 @@ class Collection(object):
             raise Exception('Provide a list of records, '
                             'or the path to a set of alignments')
 
-        if trees_dir:
-            self.read_trees(trees_dir)
+        if param_dir is not None:
+            self.read_parameters(param_dir)
 
         if not self.records:
             raise NoRecordsError(file_format, input_dir, compression)
@@ -170,17 +172,58 @@ class Collection(object):
 
             record.name = (fileIO.strip_extensions(f))
             record.fast_compute_distances()
+            record.parameters = dict(tree=record.tree.newick, name=record.name,
+                                     partitions={0:dict(distances=record.get_distance_variance_matrix().tolist())})
             records.append(record)
 
         return records
 
-    def read_trees(self, input_dir):
+    def read_parameters(self, input_dir):
         """ Read a directory full of tree files, matching them up to the
         already loaded alignments """
 
-        extensions = ['nwk', 'tree']
-        files = fileIO.glob_by_extensions(input_dir, extensions)
-        trees = [Tree.read_from_file(file_) for file_ in files]
+        for rec in self.records:
+            try:
+                with open(os.path.join(input_dir, '{}.json'.format(rec.name))) as infile:
+                    parameters = json.load(infile, parse_int=True)
+                    rec.parameters = parameters
+                    if 'partitions' in rec.parameters:
+                        rec.parameters['partitions'] = {int(k): v for (k, v) in rec.parameters['partitions'].iteritems()}
+                        try:
+                            freqs = rec.parameters['partitions'][0]['frequencies']
+                            alpha = rec.parameters['partitions'][0]['alpha']
+                            tree = rec.parameters['tree']
+                            rec.set_substitution_model('GTR' if rec.is_dna() else 'LG08')
+                            rec.set_gamma_rate_model(4, alpha)
+                            rec.set_frequencies(freqs)
+                            if rec.is_dna():
+                                rec.set_rates(result['partitions'][0]['rates'], 'ACGT')
+                            rec.initialise_likelihood(str(tree))
+                        except KeyError:
+                            pass
+
+                        try:
+                            dists = rec.parameters['partitions'][0]['distances']
+                            rec.set_distance_matrix(dists)
+                        except KeyError:
+                            pass
+
+            except IOError:
+                continue
+
+    def write_parameters(self, output_dir):
+        if not os.path.exists(output_dir):
+            try:
+                os.makedirs(output_dir)
+            except IOError as err:
+                sys.stderr.write(err.message)
+                raise err
+                return
+
+        for rec in self.records:
+            with open(os.path.join(output_dir, '{}.json'.format(rec.name)), 'w') as outfile:
+                parameters = dict((key, val) for (key, val) in rec.parameters.iteritems())
+                json.dump(rec.parameters, outfile, indent=4, separators=(',', ': '))
 
     def fast_calc_distances(self):
         """ Calculates within-alignment pairwise distances and variances for every
@@ -203,19 +246,26 @@ class Collection(object):
             pbar.update(i)
         pbar.finish()
 
-    def calc_pll_trees(self, threads=1):
+    def calc_trees(self, threads=1, indices=None):
         """ Use pllpy to calculate maximum-likelihood trees
         :return: void
         """
-        pbar = setup_progressbar('Calculating ML trees', len(self))
+
+        if indices is None:
+            indices = range(len(self))
+        else:
+            indices = indices
+
+        pbar = setup_progressbar('Calculating ML trees', len(indices))
         pbar.start()
-        for i, rec in enumerate(self.records):
+
+        for i, rec in enumerate(self[i] for i in indices):
             if rec.is_dna():
                 model = 'GTR'
             else:
                 model = 'LGX'
             result = rec.pll_optimise('{}, {} = 1 - {}'.format(model, rec.name, len(rec)), rec.tree.newick,
-                                                               nthreads=threads, seed=PLL_RANDOM_SEED)
+                                      nthreads=threads, seed=PLL_RANDOM_SEED)
             freqs = result['partitions'][0]['frequencies']
             tree = result['tree']
             alpha = result['partitions'][0]['alpha']
@@ -225,10 +275,14 @@ class Collection(object):
             if rec.is_dna():
                 rec.set_rates(result['partitions'][0]['rates'], 'ACGT')
             rec.initialise_likelihood(tree)
+            rec.compute_distances()
+            result['partitions'][0]['distances'] = rec.get_distance_variance_matrix().tolist()
+            result['name'] = rec.name
+            rec.parameters = result
             pbar.update(i)
         pbar.finish()
 
-    def get_tree_distance_matrix(self, metric, **kwargs):
+    def get_inter_tree_distances(self, metric, **kwargs):
         """ Generate a distance matrix from a fully-populated Collection """
         return DistanceMatrix(self.trees, metric, **kwargs)
 
@@ -280,7 +334,7 @@ class Concatenation(object):
 
     @lazyprop
     def distances(self):
-        return [self.collection.records[i].get_distances() for i in self.indices]
+        return [self.collection.records[i].get_distance_variance_matrix() for i in self.indices]
 
     @lazyprop
     def datatypes(self):
@@ -386,6 +440,7 @@ class Concatenation(object):
         dv, gm, lab, tree_string = self._get_tree_collection_strings(scale)
 
         import tree_collection
+
         output_tree, score = tree_collection.compute(dv, gm, lab, tree_string,
                                                      niters, keep_topology,
                                                      quiet)
@@ -450,7 +505,7 @@ class Scorer(object):
     def records(self):
         return self.collection.records
 
-    def get_minsq_partition(self, partition_list):
+    def get_minsq_partition(self, partition):
         """ Calculates concatenated trees for a list of Partitions """
         index_tuples = partition.get_membership()
         return self._get_minsq_index_tuple_list(index_tuples)
@@ -505,8 +560,9 @@ class Scorer(object):
             return self.minsq_cache[index_tuple]
         except KeyError:
             conc = self.concatenate(index_tuple)
-            result = self.minsq_cache[index_tuple] = conc.minsq_tree()
-            return result
+            tree, sse = self.minsq_cache[index_tuple] = conc.minsq_tree()
+            n_tips = len(tree)
+            return dict(tree=tree, sse=sse, fit=sse / (2 * (n_tips - 2) * (n_tips - 3)))
 
     def concatenate(self, index_tuple):
         """ Returns a Concatenation object that stitches together
@@ -536,9 +592,9 @@ class Scorer(object):
         """ Gets records by their index, contained in the index_tuple """
         return [self.records[n] for n in index_tuple]
 
-    def score(self, partition, criterion):
+    def get_results(self, partition, criterion):
         """
-        Return the score for a partition - either the sum of log likelihoods,
+        Return the results for scoring a partition - either the sum of log likelihoods,
         or the total min squares dimensionless fit index
         :param partition: Partition object
         :param criterion: either 'minsq' or 'lnl'
@@ -547,4 +603,34 @@ class Scorer(object):
         optioncheck(criterion, ['lnl', 'minsq'])
         results = (self.get_lnl_partition(partition) if criterion == 'lnl'
                    else self.get_minsq_partition(partition))
-        return results  # TODO: how to sum scores from this?
+        return results
+
+    def get_likelihood(self, partition):
+        """
+        Return the sum of log-likelihoods for a partition.
+        :param partition: Partition object
+        :return: score (float)
+
+        """
+        results = self.get_results(partition, 'lnl')
+        return math.fsum(x['likelihood'] for x in results)
+
+    def get_sse(self, partition):
+        """
+        Return the sum of squared errors score for a partition
+        :param partition: Partition object
+        :return: score (float)
+        """
+        results = self.get_results(partition, 'minsq')
+        return math.fsum(x['sse'] for x in results)
+
+    def get_fit(self, partition):
+        """
+        Return the dimensionless fit index for a partition
+        (sum of sq err / (2*(T-2)*(T-3), where T is the number of tips).
+        in the tree.
+        :param partition: Partition object
+        :return: score (float)
+        """
+        results = self.get_results(partition, 'minsq')
+        return math.fsum(x['fit'] for x in results)
